function UIBase() {
  var self = this;

  this.tapEvent = ('ontouchstart' in document.documentElement) ? 'fastTap' : 'click';
  // AZ: 20110329 I know this is a hack but events need to be revisited in earnest - there are many issues surrounding them
  this.slowTapEvent = ('ontouchstart' in document.documentElement) ? 'tap' : 'click';
  $('.additional-info').bind('click', function() { return false; });

  $('#popup-overlay > section').prepend("<nav class='close'></nav>");
  $('#popup-overlay > section > nav.close').bindAndStopPropagation(this.tapEvent, function() { self.closePopup(); });

  this.scrollers = {};

  this.notifier = {
    fixturesUpdated: function() {
      self._updateMatches(PUGE.fixtures);

      if (self.screen() == 'loading') {
        self.gotoFixtures();
        self.scrollToFirstLiveMatch();
        if(!localStorage.getItem('hideHowToPlay')) {
          localStorage.setItem('hideHowToPlay', true);
          self.popup('how-to-play');
        }
      }
    },

    matchUpdated: function(match) {
      self._updateMatch(match);
    },

    userUpdated: function() {
      self.setUser(PUGE.user);
    },

    entryOutcome: function(entry) {
      self.popupGoalAlert(entry);
    },

    playHistoryUpdated: function(playHistory) {
      self.updatePlayHistory(playHistory);
    },

    leaderboardUpdated: function(leaderboard) {
      self.updateLeaderboard(leaderboard);
    }

  };
}

UIBase.prototype.screen = function(name) {
  var bodyElement = $('body');
  if (name) {
    var screen_name = name + '-screen';

    var classAttribute = bodyElement.attr("class");
    var currentScreenName;
    if (classAttribute) {
      currentScreenName =  classAttribute.match(/\w*-screen/);
      if (currentScreenName) {
        bodyElement.removeClass(currentScreenName[0]);
      }
    }

    bodyElement.addClass(screen_name);
    this._updateLoggedInState(bodyElement);

    if(ANALYTICS) {
      ANALYTICS.trackPageView('/' + name);
    }

    this._updatePUGE();
  }

  return bodyElement.attr('class').match(/\b(\w+)-screen\b/, '')[1];
};

UIBase.prototype._updateLoggedInState = function(bodyElement) {

  var userStatus = 'logged-in';
  if (this.user && !bodyElement.hasClass(userStatus)) {
    bodyElement.addClass(userStatus);
  } else if (!this.user) {
    bodyElement.removeClass(userStatus);
  }

};

UIBase.prototype.popup = function(name) {
  $('body').addClass('popup');
  $('#popup-overlay').attr('class', name + '-popup');
};

UIBase.prototype.closePopup = function() {
  $('body').removeClass('popup');
  $('#popup-overlay').attr('class', '');
};




UIBase.prototype.goBack = function() {
  if ($('body').hasClass('popup')) {
    this.closePopup();
    return true;
  } else {
    switch(this.screen()) {
      case 'login':
        this.login.performCancel();
        return true;
      case 'soccer1up':
        // TODO DIEGO: This line is sending the same event no matter if the back button is software or hardware.
        // Fix it to send different events for each button. SOCCSP-274
        ANALYTICS.trackEvent('Shirts', 'Tap', 'BackButton', 0); // Software Back button on Shirts Page
        this.gotoFixtures();
        return true;
      case 'mypage':
        ANALYTICS.trackEvent('MyPage', 'Tap', 'BackButton', 0); // Software Back button on My Page Page
        this.gotoFixtures();
        return true;
      case 'more':
        ANALYTICS.trackEvent('More', 'Tap', 'BackButton', 0); // Software Back button on More Page
        this.gotoFixtures();
        return true;
      case 'playhistory':
        ANALYTICS.trackEvent('PlayHistory', 'Tap', 'BackButton', 0); // Software Back button on Play History Page
        this.gotoFixtures();
        return true;
      case 'leaderboard':
        ANALYTICS.trackEvent('Leaderboard', 'Tap', 'BackButton', 0); // Software Back button on Leaderboard Page
        this.gotoFixtures();
        return true;
    }
  }

  return false;
};

UIBase.prototype.gotoFixtures = function() {
  this._collapseFooter();
  this.fixtures.unselectMatch();
  this.screen('fixtures');
  this.updateScroller('fixtures');
};

UIBase.prototype.gotoSoccer1up = function(match) {
  this._collapseFooter();
  this.fixtures.selectMatch(match);
  this.soccer1up.setupMatch(match);
  this.soccer1up.closeSelectedPlayer();
  this.screen('soccer1up');
};

UIBase.prototype.gotoMyPage = function() {
  this._collapseFooter();
  this.screen('mypage');
};

UIBase.prototype.gotoMore = function() {
  this._collapseFooter();
  this.screen('more');
  this.updateScroller('more');
};

UIBase.prototype.gotoPlayHistory = function() {
  this._collapseFooter();
  this.screen('playhistory');
};

UIBase.prototype.gotoLeaderboard = function() {
  this._collapseFooter();
  this.screen('leaderboard');
};

UIBase.prototype.gotoLogin = function(afterLogin) {
  this.login.nextScreen = this.screen();
  this.login.afterLogin = afterLogin;
  this.screen('login');
};

UIBase.prototype.popupSpoils = function(game) {
  this.spoilsPopup.setupGame(game);
  this.popup('spoils');
};

UIBase.prototype.popupGoalAlert = function(entry) {
  this.goalAlertPopup.setupEntry(entry);
  this.popup('goal-alert');
};

UIBase.prototype.popupPlayConfirmation = function(entry) {
  this.playConfirmationPopup.setupEntry(entry);
  this.popup('play-confirmation');
};

UIBase.prototype.popupPlayError = function(title, error, url) {
  this.playErrorPopup.setupError(title, error, url);
  this.popup('play-error');
};

UIBase.prototype.setUser = function(user) {
  this.user = user;
  this.toolbar.setupUser(user);
  this.mypage.setupUser(user);
  this.playHistory.setupUser(user);
  this.leaderboard.setupUser(user);
  this.payment.setAccountBalance(user);
  this.soccer1up._updateMyPick();
};

UIBase.prototype.updatePlayHistory = function(playHistory) {
  this.playHistory.update(playHistory);
};

UIBase.prototype.updateLeaderboard = function(leaderboard) {
  this.leaderboard.update(leaderboard);
};

UIBase.prototype._updateMatches = function(matches) {
  this.fixtures.syncMatches(matches);
};

UIBase.prototype._updateMatch = function(match) {
  this.fixtures.updateMatch(match);

  if (this.soccer1up.match && match.id == this.soccer1up.match.id) {
    this.soccer1up.setupMatch(match);
  }  
};

UIBase.prototype._collapseFooter = function() {
  $("footer.expanded").removeClass("expanded").addClass("collapsed");
};

UIBase.prototype.updateScroller = function(screen) {
  if(!this.scrollers[screen]) {
    this.scrollers[screen] = new IScroll($('#' + screen + '-screen .wrapper')[0]);
  }
  this.scrollers[screen].refresh();

  return this.scrollers[screen];
};

UIBase.prototype.disconnected = function(notConnected) {
  if (notConnected) {
    $('.disconnected').show();
  }
  else {
    $('.disconnected').hide();
  }
};

UIBase.prototype._updatePUGE = function() {
  switch(this.screen()) {
    case 'loading':
    case 'fixtures':
      PUGE.refreshFixtures();
      break;
    case 'soccer1up':
      PUGE.refreshMatch(this.soccer1up.match.id);
      break;
    case 'playhistory':
      if (! this.playHistory.isCurrentlyRefreshing()) {
        PUGE.refreshPlayHistory();
      }
      break;
    case 'leaderboard':
      PUGE.refreshLeaderboard();
      break;
  }

  PUGE.refreshUser();
};

UIBase.prototype.gotoPuge = function(path) {
  // this is lame because PhoneGap opens the url in the app, rather than the browser.  So we have to
  // screw around with the dom etc to make the right thing happen

  PUGE.redirectorUrl(path, function(url) {
//    window.open(url);
  
    var anchor = document.createElement('a');
    anchor.href = url;
    
    $(anchor).trigger('click');
  });
};

UIBase.prototype.ensureLoggedIn = function(fn) {
  if (PUGE.user) {
    fn();
  } else {
    this.gotoLogin(fn);
  }
};

UIBase.prototype.hideKeyboard = function() {
  if (window.KeyBoard) {
    window.KeyBoard.hideKeyBoard();
  }
};

UIBase.prototype.scrollToFirstLiveMatch = function() {
  if (this.scrollers.fixtures) {
    this.scrollers.fixtures.scrollToElement('#fixtures-screen li[data-match-status="live"]', 0);
  }
};

UIBase.prototype.facebookLogin = function() {
  facebook = new FacebookConnect();
  facebook.login();
  credentials = facebook.credentials;
  console.log("facebook login!");
  console.log(credentials);
};

UIBase.prototype.facebookConnectionError = function(error) {
  this.closePopup();
  if (error) { console.log(error); }
  alert('Unknown error occurred connecting to Facebook');
};

UIBase.prototype.facebookCancel = function() {
  this.closePopup();
};

UIBase.prototype.facebookComplete = function(user_id, access_token) {
  UI.popup('processing');

  var credentials = {
    facebook_user_id: user_id,
    access_token: access_token
  };

  console.log(user_id);
  console.log(access_token);
  console.log("details received!");

  PUGE.login(credentials, function(user, errorObj) {

    if (user) {

      // We need to set PUGE.credentials.login so when we click
      // links from 'My Page', the Users::Redirector works
      //
      PUGE.credentials.login = user.login;

      UI.screen(UI.login.nextScreen);
      UI.closePopup();

      if (UI.login.afterLogin) {
        UI.login.afterLogin();
      }
    }
    else if (errorObj.status == 422) {
      UI.screen(UI.login.nextScreen);
      UI.provideUniqueLoginPopup.setCredentials(credentials);
      UI.popup('provide-unique-login');
    }
    else {
      UI.closePopup();
      alert(errorObj.error);
    }
  });
};

UI = new UIBase();

$(function() {
  PUGE.notifiee = UI.notifier;

  $('#header').bindAndStopPropagation(UI.tapEvent, function() {
    window.location.reload(true);
  });

  if(PUGE.credentials) {
    if(PUGE.credentials.facebook_user_id) {
      // must wait for phonegap to be ready to call to the native java class
      document.addEventListener("deviceready", function() { UI.facebookLogin(); });
    } else {
      PUGE.login(PUGE.credentials);
    }
  }

  function updatePUGE() {
    UI._updatePUGE();
  }

  if(Config.refreshEnabled) {
    window.setInterval(updatePUGE, Config.refreshInterval);
  }

  updatePUGE();
});

// Prevent the whole screen to scroll when dragging elements outside of the scroller (ie:header/footer).
// If you want to use iScroll in a portion of the screen and still be able to use the native scrolling, do *not* preventDefault on touchmove.

UI.scrollDisabled = false;
document.addEventListener('touchmove', function(e) {
  if(UI.scrollDisabled) {
    e.preventDefault();
  }
}, false);