function FixturesUI(screenElement) {
  var self = this;

  this.screenElement = screenElement;

  this.elements = {
    scroller: $('.scroller', screenElement)
  };

  // setup matchTemplate
  this.matchTemplate = screenElement.find('li.template').remove().removeClass('template').html();

  this._setupWobble(screenElement.find("nav.sports > ul, nav.leagues > ul"));
  this._setupLiveMatchHandler();


}

FixturesUI.prototype._findMatchElement = function(match) {
  return this.elements.scroller.find('[data-match="' + match.id + '"]');
};

FixturesUI.prototype.selectMatch = function(match) {
  this.unselectMatch();

  this._findMatchElement(match).addClass('selected');
};

FixturesUI.prototype.unselectMatch = function() {
  $('#scroller .selected', this.screenElement).removeClass('selected');
};

FixturesUI.prototype._setupWobble = function(elements) {
  var origin = { x:0 }, dest = { x:0 };

  elements
    .bind('touchstart', function(e) {
      var touch = e.targetTouches[0];
      origin.x = touch.pageX;
    })
    .bind('touchmove', function(e) {
      var touch = e.targetTouches[0];
      dest.x = touch.pageX;

      var wobbleClass = (dest.x > origin.x) ? 'wobble-left' : 'wobble-right';
      $(this).attr('class', wobbleClass);
    })
    .bind('touchend', function() {
      $(this).attr('class','wobble-center');
    });
};

FixturesUI.prototype._matchesAreSynced = function(matches) {
  var children = this.elements.scroller.get(0).children; 

  if (children.length != matches.length) {
    return false;
  }

  for (var i in matches) {
    if (!children[i] || children[i].getAttribute('data-match') != matches[i].id) {
      return false;
    }
  }

  return true;
};

FixturesUI.prototype._addRoundedCorners = function() {
  // crappy hack, if you can figure it out in CSS, go for it
  this.elements.scroller.find('[data-match-status=final]').first().addClass('rounded-top');
  this.elements.scroller.find('[data-match-status=final]').last().addClass('rounded-bottom');
  this.elements.scroller.find('[data-match-status=next]').first().addClass('rounded-top');
  this.elements.scroller.find('[data-match-status=next]').last().addClass('rounded-bottom');
};

FixturesUI.prototype._addDividers = function() {
  this.elements.scroller.find('[data-match-status=next]').first().before("<li class='divider'></li>");

};

FixturesUI.prototype.syncMatches = function(matches) {
  this.matches = matches.sort(this._matchStatusSort);

  if (!this._matchesAreSynced(matches)) {
    this.elements.scroller.html('');  // clear them, crap hack

    for (var i in matches) {
      this._createMatchElement(matches[i]);
      this.updateMatch(matches[i]);
    }


    this._addRoundedCorners();
    this._addDividers();
  }
  else    // just update matches
  {
    for (var j in matches) {
      this.updateMatch(matches[j]);
    }
  }
};

FixturesUI.prototype._matchStatusSort = function(a, b) {
  var order = [ 'final', 'live', 'next'];
  return order.indexOf(a.status) - order.indexOf(b.status);
};

FixturesUI.prototype._setupLiveMatchHandler = function() {
  var self = this;

  // simonr 20110104 tap event doesn't work because of iscroll
  this.elements.scroller.bindAndStopPropagation(UI.tapEvent, function(event) {
    var matchElement = $(event.target).closest('[data-match]').get(0);
    var findId = matchElement.getAttribute('data-match');

    for (var i in self.matches) {
      var match = self.matches[i];
      // jd - allow click into live or next status matches, but not final.
      if (match.id == findId && match.status != 'final' && match.soccer1upStatus() == 'available') {
        if ($(event.target).closest('.spoils').get(0)) {
          UI.popupSpoils(self.matches[i].games.soccer1up);
          ANALYTICS.trackEvent('Fixture', 'Tap', 'Spoils', 0); // Spoils button on Fixtures page
        } else {
          UI.gotoSoccer1up(self.matches[i]);
        }
      }
    }
  });
};

FixturesUI.prototype._interpolateMatch = function(match, player) {
  return Templater.interpolate(this.matchTemplate, function(key) {
    switch (key) {
      case 'match-time':      return match.matchTime;
      case 'home-score':      return match.homeScore;
      case 'home-team-name':  return match.homeTeam.shortName;
      case 'away-score':      return match.awayScore;
      case 'away-team-name':  return match.awayTeam.shortName;
      case 'start-date':      return match.startDate;
      case 'start-time':      return match.startTime;
    }

    if (player) {
      switch (key) {
        case 'shirt-number':            return player.shirtNumber;
        case 'player-name':             return player.name;
        case 'player-team-short-name':  return player.team.shortName;
      }
    }

    if (match.games.soccer1up) {
      switch (key) {
        case 'spoils-cash':     return '&pound; ' + match.games.soccer1up.cash;
        case 'spoils-points':   return 'Pts: ' + match.games.soccer1up.points;
      }
    }
  });
};

FixturesUI.prototype._createMatchElement = function(match) {
  var newElement = document.createElement('li');
  newElement.setAttribute('data-match', match.id);
  this.elements.scroller.append(newElement);

  return newElement;
};

FixturesUI.prototype.updateMatch = function(match) {

  var player = PUGE.user && PUGE.user.soccer1upPick(match);

  this._findMatchElement(match)
          .html(this._interpolateMatch(match, player))
          .attr('data-match-status', match.status)
          .attr('data-game-status', match.soccer1upStatus())
          .attr('data-player-status', player ? 'selected' : 'unselected');

  if(this.scrollerTimer) { clearTimeout(this.scrollerTimer); }
  this.scrollerTimer = setTimeout(function() { UI.updateScroller('fixtures'); }, 200);
};

$(function() {
  UI.fixtures = new FixturesUI($('#fixtures-screen'));
});
