function GoalAlertPopup(element) {
  this.elements = {
    body: element,
    message: element.find('h1'),
    subMessage: element.find('h2'),
    pick: element.find('.pick'),
    goalScorerTeam: element.find('h4'),
    goalScorer: element.find('.player'),
    goalScorerName: element.find('.player span'),
    goalScorerShirtNumber: element.find('.player em'),
    skillRankingPoints: element.find('.winnings .skill'),
    cash: element.find('.winnings .cash')
  };
}

GoalAlertPopup.prototype.setupEntry = function(entry) {
  this._setupStatus(entry);
  this._setupSpoils(entry);
  this._setupPickMessage(entry);
  this._setupScorerDetails(entry);
};

GoalAlertPopup.prototype._setupSpoils = function(entry) {
  var spoils = entry.spoils;

  this.elements.cash.text("");
  this.elements.skillRankingPoints.text("");

  if(spoils){
    if(spoils.cash){
      this.elements.cash.html("Cash: &pound;" + entry.spoils.cash);
    }
    if(spoils.skillRankingPoints){
      this.elements.skillRankingPoints.text("Skill pts: +" + entry.spoils.skillRankingPoints);
    }
  }
};

GoalAlertPopup.prototype._setupPickMessage = function(entry) {
  this.elements.pick.text([
    'Your 1UP pick: ',
    entry.prediction.team.shortName,
    ', ',
    entry.prediction.player.name,
    ' (',
    entry.prediction.player.shirtNumber,
    ')'
  ].join(''));
};

GoalAlertPopup.prototype._setupScorerDetails = function(entry) {
  this.elements.goalScorerName.text(entry.goal.player.name);
  this.elements.goalScorerTeam.text(entry.goal.team.name + " Scores!");
  this.elements.goalScorerShirtNumber.text(entry.goal.player.shirtNumber);
  this.elements.goalScorer.attr("class", "player " + entry.goal.team.shortName);
};

GoalAlertPopup.prototype._setupScoringTeam = function(entry) {
  console.log(entry);
  if(entry.goal.ownGoal) {
    message = "Kick an Own Goal!";
  } else {
    message = "Score!";
  }
  
  this.elements.scoringTeam.text(entry.goal.team.name + ' ' + message);
};

GoalAlertPopup.prototype._setupStatus = function(entry) {
  var outcomeStatus = entry.outcome.status;
  this.elements.body.attr('data-status', outcomeStatus);

  var messages = ['', ''];

  switch (outcomeStatus) {
    case 'won':
      messages = ['Awesome!', 'You win!'];
      break;

    case 'runner_up':
      messages = ['Right team', 'Wrong player!'];
      break;

    case 'lost':
      too_bad = 'Too Bad' + (entry.goal.ownGoal && entry.prediction.player.id == entry.goal.player.id ? ', Own Goal!' : '!');
      messages = [too_bad, 'Try again?'];
      break;
  }

  this.elements.message.text(messages[0]);
  this.elements.subMessage.text(messages[1]);
};

$(function() {
  UI.goalAlertPopup = new GoalAlertPopup($('#goal-alert-popup'));
});
