function LoginUI(element) {
  var self = this;

  this.loginField = element.find("input[name=login]");
  this.passwordField = element.find("input[name=password]");
  this.loginButton = element.find("div.login > .login-button");
  this.registerButton = element.find('div.register > input[type=submit]');
  this.cancelButton = element.find('#login-cancel');
  this.forgottenPassword = element.find('.forgotten-password a');
  this.errors = element.find('.errors');
  this.resendActivationLink = element.find('a.resend-activation');
  this.fbLogin = element.find('div.login > .fb-login');
  this.loginField.bindAndStopPropagation('keydown', function() { self._hideErrors(); });
  this.passwordField.bindAndStopPropagation('keydown', function() { self._hideErrors(); });

  this.loginButton.bindAndStopPropagation(UI.tapEvent, function() { self.performLogin(); });
  this.registerButton.bindAndStopPropagation('click', function() { self.performRegister();  });
  this.resendActivationLink.bindAndStopPropagation(UI.tapEvent, function() { self.resendActivation(); });

  this.cancelButton.bindAndStopPropagation('click', function() { self.performCancel(); });
  this.forgottenPassword.attr('href', Config.pugeServer + '/uk/en/mobile/password_reset/new');
  this.fbLogin.bindAndStopPropagation('click', function() { self.facebookLogin(); });

  this.afterLogin = null;
  this._errorObj = null;
}

LoginUI.prototype.performLogin = function() {

  var self = this;
  self.loginButton.attr('disabled', 'disabled');

  ANALYTICS.trackEvent('Login', 'Tap', 'Login', 0); // Login button on Login screen

  // Some phones add a space to the end of autocompleted words. Avoid a confusing
  // failed login below.
  self.loginField.val(this.loginField.val().replace(/\s+$/, ''));

  PUGE.login({login: this.loginField.val(), password: this.passwordField.val()}, function(user, errorObj) {

    self.loginButton.removeAttr('disabled');

    if (user) {
      document.activeElement.blur();
      self.loginField.val('');
      self.passwordField.val('');
      UI.screen(self.nextScreen);
      UI.hideKeyboard();

      if (self.afterLogin) {
        self.afterLogin();
      }
    } else {
      self._errorObj = errorObj;
      self._displayError(self._errorObj);
    }
  });
};

LoginUI.prototype.performRegister = function() {
  ANALYTICS.trackEvent('Login', 'Tap', 'RegisterNow', 0); // Register button on Login Screen
  UI.screen('register');
};

LoginUI.prototype.performCancel = function() {
  ANALYTICS.trackEvent('Login', 'Tap', 'Cancel', 0); // Cancel button on login screen
  UI.screen(this.nextScreen);
  this._hideErrors();
  this._removeSending();
};

LoginUI.prototype.resendActivation = function() {
  var self = this;

  this.errors.addClass('sending');

  $.ajax({
    type: 'POST',
    url: PUGE.api.resendActivation,
    data: { email: self._errorObj.email },
    dataType: 'json',
    success: function() {
      alert('Email sent to ' + self._errorObj.email + '. Thanks!');
      self._hideErrors();
    },
    error: function() {
      alert('Error while sending the email.');
      self._hideErrors();
    }
  });
};

LoginUI.prototype._displayError = function(errorObj) {
  this._hideErrors();
  this.errors.addClass('expanded status-' + errorObj.status);

  if (errorObj.status == 423) {
    // User account exists, but is not verfied
    ANALYTICS.trackEvent('Login', 'Notification', 'AccountNotActivated', 0);
  }
};

LoginUI.prototype._hideErrors = function() {
  this.errors[0].className = 'errors';
};

LoginUI.prototype.facebookLogin = function() {
  ANALYTICS.trackEvent('Login', 'Tap', 'Facebook', 0);
  UI.facebookLogin();
};

LoginUI.prototype._removeSending = function() {
  $(".sending").removeClass("sending");
};

$(function() {
  UI.login = new LoginUI($('#login-screen'));
});
