function PlayHistoryUI(element) {
  var self = this;
  this.element = element;
  this.loginField = element.find('.details > h2 > em');
  this.prev = element.find('.pagination > .prev');
  this.next = element.find('.pagination > .next');

  self.content = element.find('.playhistory-results tbody');
  element.find('.pagination').delegate("div[data-page]", "click", function(evt) {
    self.setStartedRefreshing();
    PUGE.refreshPlayHistory($(evt.target).attr("data-page"));
    // Scroll to the top of the page if and only if the prev/next button has been clicked
    UI.scrollers.playhistory.scrollToTop();
  });

}

PlayHistoryUI.prototype.update = function(playHistory) {
  var self = this;
  self.content.html("");
  self._appendMatches(playHistory.matches, function(match) {
    self.baseDate = match.scheduled_start;
    self._appendGames(match.games, function(entries) {
      self._appendEntries(entries);
    });
  });
  self._setupPagination(playHistory.page);
  UI.updateScroller('playhistory');
  self.setFinishedRefreshing();
};


PlayHistoryUI.prototype._setupPagination = function(page) {
  // Navigation is a bit weird here as going forward is going back in time.
  this._setupPageNav(this.prev, page.next_page);
  this._setupPageNav(this.next, page.previous_page);
};

PlayHistoryUI.prototype._setupPageNav = function(navElement,pageNo) {
  if (pageNo !== null) {
    navElement.attr("data-page",pageNo);
  } else {
    navElement.removeAttr("data-page");
  }
};

PlayHistoryUI.prototype._appendMatches = function(matches, callback) {
  var self = this;
  for(var i = 0; i < matches.length; i++) {
    var match = matches[i];
    self.content.append("<tr class='match'><td>" + Util.toTimeWithOffset(match.scheduled_start).strftime("%b %d") + "</td><td>" + match.description + "</td><td></td></tr>");
    callback(match);
  }
};

PlayHistoryUI.prototype._appendGames = function(games, callback) {
  var self = this;
  for(var i in games) {
    var pools = games[i].pools;
    for(var j in pools) {
      var pool = pools[j];
      var goal = pool.goal;
      if (pool.goal) {
        self.content.append("<tr class='scorer'><td>" + Util.formatRelativeDate(self.baseDate, goal.game_time) + "</td><td>" + goal.scorer + "</td><td></td></tr>");
      }
      callback(pool.entries);
    }
  }
};

PlayHistoryUI.prototype._appendEntries = function(entries) {
  var self = this;
  for (var i in entries) {
    var entry = entries[i];
    self.content.append("<tr class='entry-" + entry.outcome + "'><td>" + Util.formatRelativeDate(self.baseDate, entry.entered_at) + "</td><td>" + entry.prediction + "</td><td><p>" + entry.outcome + "</p></td></tr>");
    if (entry.spoils) {
      self.content.append("<tr><td></td><td>Won - Skill ranking pts:</td><td>" + Util.commas(entry.spoils.skillRankingPoints) + "</td></tr>");
    }
  }
};


PlayHistoryUI.prototype.setupUser = function(user) {
  if (user) {
    this.loginField.text(user.login);
  }
};

PlayHistoryUI.prototype.setStartedRefreshing = function() {
  $('#playhistory-screen').addClass('refreshing');
};

PlayHistoryUI.prototype.setFinishedRefreshing = function() {
  $('#playhistory-screen').removeClass('refreshing');
};

PlayHistoryUI.prototype.isCurrentlyRefreshing = function() {
  return $('#playhistory-screen').hasClass('refreshing');
};

$(function() {
  UI.playHistory = new PlayHistoryUI($('#playhistory-screen'));
});

