function ProvideUniqueLoginPopupUI(element) {
  var self = this;

  this.element = element;

  this.title = element.find('h1');
  this.errors = element.find('.errors');
  this.error  = element.find('.errors > div');

  this.acceptButton = element.find('.buttons .accept');
  this.cancelButton = element.find('.buttons .cancel');
  this.loginField = element.find("input[name=login]");

  this.loginField.bindAndStopPropagation('keydown', function() {
    self._hideErrors();
    self._removeSending();
  });
  this.acceptButton.bindAndStopPropagation(UI.tapEvent, function() {
    self.sendLogin();
    self._removeSending();
  });
  this.cancelButton.bindAndStopPropagation(UI.tapEvent, function() {
    self.proceedToFixtures();
    self._removeSending();
  });

  this.credentials = {};
}

ProvideUniqueLoginPopupUI.prototype.sendLogin = function() {
  var self = this;

  ANALYTICS.trackEvent('Nickname', 'Tap', 'OK', 0);

  this.errors.addClass('sending');

  $.ajax({
    type: 'POST',
    url: PUGE.api.addLogin,
    data: {
      login: $(self.loginField).val(),
      facebook_user_id: self.credentials.facebook_user_id,
      games: PUGE.interestingGames()
    },
    dataType: 'json',
    success: function(user) {
      PUGE.setCredentialsAndUser($.extend(self.credentials, { login: user.login }), user);

      if (UI.login.afterLogin) {
        UI.login.afterLogin();
      } else {
        self.proceedToFixtures();
      }
    },
    error: function(response) {
      self._displayError(Util.createErrorObject(response));
    }
  });
};

ProvideUniqueLoginPopupUI.prototype.setCredentials = function(credentials) {
  this.credentials = credentials;
};

ProvideUniqueLoginPopupUI.prototype.proceedToFixtures = function() {
  ANALYTICS.trackEvent('Nickname', 'Tap', 'Close', 0);
  UI.closePopup();
  UI.gotoFixtures();
  this._hideErrors();
};

ProvideUniqueLoginPopupUI.prototype._displayError = function(errorObj) {
  this._hideErrors();
  this.errors.addClass('expanded status-' + errorObj.status);

  var error_message;

  error_message = errorObj.error;

  // Whilst the response from PUGE is correct, we want a slightly different message
  // for 'Login has already been taken' because we wish to present the message as
  // 'Nickname has already been taken'
  //
  if (error_message == 'Login has already been taken') {
    error_message = 'Nickname has already been taken';
  }

  this.error.html(error_message);

  ANALYTICS.trackEvent('Login', 'Notification', errorObj.error, 0);
};

ProvideUniqueLoginPopupUI.prototype._hideErrors = function() {
  this.errors[0].className = 'errors';
};

ProvideUniqueLoginPopupUI.prototype._removeSending = function() {
  $(".sending").removeClass("sending");
};


$(function() {
  UI.provideUniqueLoginPopup = new ProvideUniqueLoginPopupUI($("#provide-unique-login-popup"));
});

