Soccer1upUI = function(screenElement) {
  var self = this;

  this.screenElement = screenElement;
  this.scorer = null;
  this.match = null;

  this.element('a.swap-team.away').bindAndStopPropagation(UI.tapEvent, function() { self.showHomeTeam(); });
  this.element('a.swap-team.home').bindAndStopPropagation(UI.tapEvent, function() { self.showAwayTeam(); });

  // TODO find out why live events don't work on the result of a .find()
  $('#teams li').live(UI.tapEvent, function() { self.selectPlayer($(this)); return false; });

  this.elements = {
    play: this.element('#play-button'),
    close: this.element('#close'),
    playFree: this.element('#play-now-free'),
    playCash: this.element('#play-now-cash'),
    playSMS: this.element('#play-now-sms')
  };

  this.elements.play.bindAndStopPropagation(UI.tapEvent, function() { self.showPlaceEntry(); return false; });
  this.elements.close.bind(UI.slowTapEvent, function() { self.closeSelectedPlayer(); return false; });
  this.elements.playFree.bind('click', function(event) {
    $(this).addClass("sending");
    event.stopPropagation();
    self.placeEntry('points');
    return false;
  });
  this.elements.playCash.bind('click', function(event) {
    $(this).addClass("sending");
    event.stopPropagation();
    self.placeEntry('cash');
    return false;
  });
  this.elements.playSMS.bind('click', function() {
    $(this).addClass("sending");
    event.stopPropagation();
    self.placeEntry('sms');
    return false;
  });

  this.templates = {
    player: {
      inner: this.element("ul#player-template li").remove().html(),
      goal: this.element('#soccer1up-templates .goal').remove().get(0).outerHTML,
      redYellowCard: this.element('#soccer1up-templates .red-yellow-card').remove().get(0).outerHTML,
      redCard: this.element('#soccer1up-templates .red-card').remove().get(0).outerHTML,
      yellowCard: this.element('#soccer1up-templates .yellow-card').remove().get(0).outerHTML
    }
  };

  SpoilsHelper.bindPopup(this.element('.spoils-button'), function() {
    ANALYTICS.trackEvent('Shirts', 'Tap', 'Spoils', 0); // Spoils button on Match page
    return self.match.games.soccer1up;
  });
};


Soccer1upUI.prototype.element = function(selector) {
  return this.screenElement.find(selector);
};

Soccer1upUI.prototype.showHomeTeam = function() {
  this.element('#scorer-selection').addClass('home').removeClass('away');
};

Soccer1upUI.prototype.showAwayTeam = function() {
  this.element('#scorer-selection').addClass('away').removeClass('home');
};

Soccer1upUI.prototype._updateSelectedPlayer = function() {
  var selectedPlayer = this.element('#selected-player');

  selectedPlayer.find('h2#selected-player-name').html(this.scorer.name);
  selectedPlayer.find('#big-shirt').html(this.scorer.shirtNumber);

  var selectedItem = this.element('li[data-player="' + this.scorer.id + '"]');
  var bgImage = document.defaultView.getComputedStyle(selectedItem.find('em').get(0)).backgroundImage;
  selectedPlayer.find('#big-shirt').css('background-image', bgImage);
  var shirtTextColor = selectedItem.find('em').css('color');
  selectedPlayer.find('#big-shirt').css('color', shirtTextColor);

  selectedPlayer.find('h3#position').html(this.scorer.position);
  selectedPlayer.find('#player-season-goals').html(this.scorer.seasonGoals);
  selectedPlayer.find('#player-match-goals').html(this.scorer.matchGoals);
  selectedPlayer.find('#player-fans-agree-percentage').html(this.scorer.fansAgreePercentage);

  Util.enableButton(this.elements.play, this._canPlaceEntryOnSelectedScorer());
};

Soccer1upUI.prototype._canPlaceEntryOnSelectedScorer = function() {
  return this.scorer.redCards === 0 &&
    !this._isCurrentlyPicked(this.scorer) && 
    !$('#selected-player-play').hasClass('expanded');
};

Soccer1upUI.prototype.selectPlayer = function(selectedItem) {
  var player = this.match.players[selectedItem.attr('data-player')];

  if(player.redCards === 0){
    this.scorer = player;
    this._updateSelectedPlayer();

    $('body').addClass('player-selected');
    this.element('.selected').removeClass('selected');
    selectedItem.addClass('selected');

    ANALYTICS.trackEvent('Shirts', 'Tap', 'Player', 0); // Player selected on shirts page
  }
};


Soccer1upUI.prototype.closeSelectedPlayer = function() {
  $('#selected-player-play').removeClass('expanded');
  $('body').removeClass('player-selected');
  this.element('.selected').removeClass('selected');
  Util.enableButton(this.elements.play, true);
};

Soccer1upUI.prototype.showPlaceEntry = function() {
  var self = this;

  if(self.playedBefore) {
    ANALYTICS.trackEvent('Shirts', 'Tap', 'Change', 0); // Change button on Shirts Page
  } else {
    ANALYTICS.trackEvent('Shirts', 'Tap', 'Play', 0); // Change button on Shirts Page
  }

  if (PUGE.user && !PUGE.user.msisdn) {
    $('#selected-player-play .with-phone-number').hide();
    $('#selected-player-play .without-phone-number').show();
  } else {
    $('#selected-player-play .with-phone-number').show();
    $('#selected-player-play .without-phone-number').hide();
  }

  Util.enableButton(this.elements.play, false);
  $('#selected-player-play').show().addClass('expanded');
};

Soccer1upUI.prototype._togglePlayOptions = function(enabled) {
  var options = [this.elements.playFree, this.elements.playCash, this.elements.playSMS];
  
  for(var i in options) {
    var option = options[i];
    Util.enableButton(option, enabled && !option._disabledPlayOption);
  }
};

Soccer1upUI.prototype.placeEntry = function(type) {
  var self = this;
  var entry = Entry.Factory.create(this.scorer, this.scorer.team, this.match.games.soccer1up, type);

  ANALYTICS.trackEntrySubmission(type);

  if(PUGE.user || type == 'sms') {
    self._submitEntry(self, entry);
  } else {
     UI.gotoLogin(function() { self._submitEntry(self, entry); });
  }
};

Soccer1upUI.prototype._submitEntry = function(self, entry) {
  self._togglePlayOptions(false);
  setTimeout(function() {
    if ($(".sending").length) {
      $(".sending").removeClass("sending");
      self._togglePlayOptions(true);
    }
  }, 5000);

  entry.place(
    function() {
      ANALYTICS.trackEntryCompletion(entry.type);

      self.closeSelectedPlayer();
      self._updateMyPick();

      if(!PUGE.user && entry.type == 'sms') {
        navigator.notification.alert(
            'Your entry is being processed. You will receive an SMS once confirmed', // message
            null, // callback
            'PlayUp Soccer', // title
            'OK'                  // buttonName
        );
      } else {
        UI.popupPlayConfirmation(entry);
      }
      self._togglePlayOptions(true);
    },
    function(title, errorMsg, url) {
      UI.popupPlayError(title, errorMsg, url);
      self._togglePlayOptions(true);
    }
  );
};

Soccer1upUI.prototype._addPlayersToField = function(listElement, players) {
  Util.syncDomWithObjects(
    listElement,
    listElement.find('[data-player]'),
    'data-player',
    players,
    'id',
    function(player) { return "<li data-player='" + player.id + "'></li>"; }
  );
};

Soccer1upUI.prototype._updatePlayers = function(listElement) {
  var self = this;
  var df = 1, mf = 1, fw = 1;

  listElement.find('li').each(function() {
    var element = $(this);
    var player = self.match.players[element.attr('data-player')];

    // Update the player html
    this.innerHTML = self._interpolatePlayer(player);

    // Update the selected player
    if (self.scorer && self.scorer.id == player.id) {
      self.scorer = player;
      self._updateSelectedPlayer();
    }

    // update the position
    var positionClass = null;

    switch(player.position) {
      case 'GK':
        positionClass = 'GK';
        break;

      case 'DF':
        positionClass = 'DF pos' + df;
        df++;
        break;

      case 'MF':
        positionClass = 'MF pos' + mf;
        mf++;
        break;

      case 'FW':
        positionClass = 'FW pos' + fw;
        fw++;
        break;

      default:
        positionClass = 'sub';
        break;
    }

    // TODO simonr 20110124 fix this lame hack.
    if (element.hasClass('selected')){
      positionClass += ' selected';
    }

    element.attr('class', positionClass);

    // SOCCSP-128 make it a bit more apparent the red carded player cant be selected.
    if (player.redCards !== 0)
    {
        element.addClass('red-carded-player');
    }

  });

  listElement.attr('data-formation', [df - 1, mf - 1, fw - 1].join('-'));
};

Soccer1upUI.prototype._updateEntryButtons = function(playedBefore, cashOnly, insufficientFunds) {
  this.elements.play.text(playedBefore ? 'change' : 'play');
  Util.enableButton(this.elements.playFree, !cashOnly);
  Util.enableButton(this.elements.playCash, !insufficientFunds);
  this.elements.playFree._disabledPlayOption = cashOnly; // Needed for _togglePlayOptions
};

Soccer1upUI.prototype._updateMyPick = function() {
  this.element('.my-pick').remove();

  var playedBefore = false, cashOnly = false, insufficientFunds = false;

  var entries = this._userEntriesForCurrentGame();
  for (var i in entries) {
    var entry = entries[i];

    this.element('li[data-player="' + entry.prediction.player.id + '"]').append(
      '<div class="my-pick"><em>My Pick<img src="images/icon_scorer_small.png" /></em></div>'
    );

    playedBefore = true;
    cashOnly = cashOnly || (entry.type == 'cash');
  }

  insufficientFunds = (PUGE.user && PUGE.user.cash < 100); /* assuming games will always cost £1 (100p) */
  this.playedBefore = playedBefore;
  this._updateEntryButtons(playedBefore, cashOnly, insufficientFunds);
};

Soccer1upUI.prototype._interpolatePlayer = function(player) {
  var result = Templater.interpolate(this.templates.player.inner, function(key) {
    switch (key) {
      case 'shirt-number':  return player.shirtNumber;
      case 'player-name':   return Util.formatPlayerName(player.name);
    }
  });

  if(player.matchGoals != 'None'){
    result += this.templates.player.goal;
  }

  var card = null;

  if(player.redCards > 0 && player.yellowCards > 0){
    card = this.templates.player.redYellowCard;
  }
  else if(player.redCards > 0){
    card = this.templates.player.redCard;
  }
  else if (player.yellowCards > 0){
    card = this.templates.player.yellowCard;
  }

  if(card) {
    result += card;
  }

  return result;
};

Soccer1upUI.prototype._isCurrentlyPicked = function(player) {
  var entries = this._userEntriesForCurrentGame();
  for (var i in entries) {
    if(player.id == entries[i].prediction.player.id) {
      return true;
    }
  }
  
  return false;
};

Soccer1upUI.prototype._userEntriesForCurrentGame = function() {
  if (PUGE.user && this.match && this.match.games.soccer1up) {
    return PUGE.user.entries[this.match.games.soccer1up.gameId] || [];
  }
  return [];
};

Soccer1upUI.prototype._setupTeam = function(list, switcher, matchInfoBit, team, score) {
  this._addPlayersToField(list, team.players);
  this._updatePlayers(list);

  list.attr('data-team-name', team.shortName);

  // switcher
  switcher.attr('data-team-name', team.shortName);
  switcher.find('em').html(team.shortName);

  // match info
  matchInfoBit.attr('data-team-name', team.shortName);
  matchInfoBit.find('.name').html(team.shortName);
  matchInfoBit.find('.score').html(score);
};

Soccer1upUI.prototype.setupMatch = function(match) {
  this.match = match;
  var matchTime = match.matchTime;
  if(matchTime == "Match Complete") {
    matchTime = 'Full Time';
  }
  this.element('#match-info .match-time').html(matchTime);

  this._setupTeam(this.element('#teams ul.home'), this.element('.swap-team.away'), this.element('#match-info .home'), match.homeTeam, match.homeScore);
  this._setupTeam(this.element('#teams ul.away'), this.element('.swap-team.home'), this.element('#match-info .away'), match.awayTeam, match.awayScore);
  this._updateMyPick();

  SpoilsHelper.updateInfo(this.element('.spoils-button .spoils'), this.match.games.soccer1up);
};

$(function() {
  UI.soccer1up = new Soccer1upUI($('#soccer1up-screen'));
});
