Util = {

  commas: function(integer) {
    return integer.toString().replace(/(\d)(?=(\d\d\d)+(?!\d))/g, "$1,");
  },

  money: function(cents) {
    // lame lame lame
    var whole = Math.floor(cents / 100);
    var fraction = (cents % 100).toString();

    //extra lame
    if (fraction.length == 1) {
      fraction = '0' + fraction;
    }

    return Util.commas(whole) + "." + fraction;
  },

  formatParams: function(params) { 
    var result = [];

    if (params){
      for (var k in params) {
        if (params[k] !== undefined && params[k] !== null)
        {
          result.push([encodeURIComponent(k), encodeURIComponent(params[k])].join('='));
        }
      }
    }

    return result.join('&');
  },

  formatUrl: function(url, params) {
    var formattedParams = Util.formatParams(params);
    return formattedParams.length > 0 ? [url, formattedParams].join(url.match(/\?/) ? '&' : '?') : url;
  },

  formatPlayerName: function(playerName) {
    var parts = playerName.split(' ');

    if (parts.length > 1){
      return parts[0][0] + '. ' + parts[1];
    }
    else{
      return playerName;
    }
  },

  syncDomWithObjects: function(parent, elements, attribute, objects, key, creator) {
    var domByElement = {};
    var objectsByKey = {};

    // group the elements by the attribute value
    elements.each(function() {
      domByElement[this.getAttribute(attribute)] = this;
    });

    // group the objects by key
    for (var i in objects){
      objectsByKey[objects[i][key]] = objects[i];
    }

    // delete the elements we don't want anymore
    for (var j in domByElement){
      if (!objectsByKey[j]){
        domByElement[j].parentNode.removeChild(domByElement[j]);
      }
    }

    // create the elements we don't have
    var newHtml = [];

    for (var k in objectsByKey){
      if (!domByElement[k]){
        newHtml.push(creator(objectsByKey[k]));
      }
    }

    parent.append(newHtml.join('')); 
  },

  enableButton: function(button, enabled) {
    if (enabled) {
      button.removeAttr('disabled');
    } else {
      button.attr('disabled', 'disabled');
    }
  },
  
  isButtonEnabled: function(button) {
    return !button.attr('disabled');
  },

  formatTimeComponent: function(v) {
    return v < 10 ? "0" + v : v.toString();
  },

  toTimeWithOffset: function(timeObj) {
    var date = new Date(timeObj.time_in_seconds * 1000  + timeObj.offset * 1000);
    return new Date(date.getTime() + date.getTimezoneOffset() * 60000);
  },

  formatRelativeDate: function(baseTimeObj, otherTimeObj) {
    var baseDate = this.toTimeWithOffset(baseTimeObj);
    var otherDate = this.toTimeWithOffset(otherTimeObj);
    if ( baseDate.getDate() == otherDate.getDate() ) {
      return otherDate.strftime('%I:%M%P');
    } else {
      return otherDate.strftime('%b %d %I:%M%P');
    }
  },

  formatMatchTime: function(secondsIn, maxMinutes) {
    var minutes = Math.floor(secondsIn / 60);
    var formattedMinutes = minutes >= maxMinutes ? 
                           Util.formatTimeComponent(maxMinutes) + "+" + (minutes - maxMinutes + 1) : 
                           Util.formatTimeComponent(minutes);

    return formattedMinutes + ":" + Util.formatTimeComponent(secondsIn % 60);
  },

  dateToUnixTime: function(date) {
    return Math.floor((date === undefined || date === null ? new Date() : date).getTime() / 1000);
  },

  DATE_REGEX: /^date:\s*(.+)$/mi,

  extractDateFromHeaders: function(headers) {
    var match;

    if (headers && (match = headers.match(Util.DATE_REGEX))) {
      return new Date(Date.parse(match[1]));
    } else {
      return new Date();    // just return local time; might be okay? who knows.
    }
  },

  createErrorObject: function(response) {
    return $.extend(JSON.parse(response.responseText), { status: response.status });
  }
};

IScroll.prototype.scrollToTop = function() {
  this.scrollTo(0, 0);
};


function inspect(obj, maxLevels, level){
  var str = '', type, msg;

    // Start Input Validations
    // Don't touch, we start iterating at level zero
    if(level === null) {
      level = 0;
    }

    // At least you want to show the first level
    if(maxLevels === null) {
      maxLevels = 1;
    }
    if(maxLevels < 1){
      return 'Error: Levels number must be > 0\n';
    }

    // We start with a non null object
    if(obj === null){
      return 'Error: Object NULL\n';
    }
    // End Input Validations


    // Start iterations for all objects in obj
  for (var property in obj) {
    try {
      // Show "property" and "type property"
      type = typeof(obj[property]);
      str += '- (' + type + ') ' + property +
              ( (obj[property] === null) ? (': null') : ('')) + '\n';

      // We keep iterating if this property is an Object, non null
      // and we are inside the required number of levels
      if ((type == 'object') && (obj[property] !== null) && (level + 1 < maxLevels)) {
        str += inspect(obj[property], maxLevels, level + 1);
      }
    }
    catch(err) {
      // Is there some properties in obj we can't access? Print it red.
      if (typeof(err) == 'string') {
        msg = err;
      }
      else if (err.message) {
        msg = err.message;
      }
      else if (err.description) {
        msg = err.description;
      }
      else {
        msg = 'Unknown';
      }

      str += '(Error) ' + property + ': ' + msg + '\n';
    }
  }

  return str;
}
