// based on http://code.google.com/mobile/articles/fast_buttons.html

(function($) {
  
  function parentIfText(node){
    return 'tagName' in node ? node : node.parentNode;
  }

  FastTap = function(element) {
    element.addEventListener('touchstart', this, false);
    element.addEventListener('touchmove', this, false);
    element.addEventListener('touchend', this, false);
  };

  FastTap.prototype.handleEvent = function(event) {
    switch (event.type) {
      case 'touchstart':  this.onTouchStart(event); break;
      case 'touchmove':   this.onTouchMove(event); break;
      case 'touchend':    this.onTouchEnd(event); break;
    }
  };

  FastTap.prototype.onTouchStart = function(event) {
    event.stopPropagation();

    this.startX = event.touches[0].clientX;
    this.startY = event.touches[0].clientY;
    this.target = parentIfText(event.touches[0].target);
  };

  FastTap.prototype.onTouchMove = function(event) {
    if (this.target &&
        (Math.abs(event.touches[0].clientX - this.startX) > 10 ||
         Math.abs(event.touches[0].clientY - this.startY) > 10)) {
      this.reset();
    }
  };

  FastTap.prototype.onTouchEnd = function(event) {
    event.stopPropagation();

    var target = this.target;
    this.reset();

    // Don't allow events on disabled elements
    if (target && !this._isDisabled(target)) {
      $(target).trigger('fastTap');
    }
  };

  FastTap.prototype._isDisabled = function(target) {
    var maxLevels = 5;
    // I can do the below in a one-liner, but jslint won't let me :(
    for (var levels = 0; target && levels < maxLevels; target = target.parentElement) {
      if(target.disabled) {
        return true;
      }
    }
    return false;
  };

  FastTap.prototype.reset = function() {
    this.startX = null;
    this.startY = null;
    this.target = null;
  };

  var makeLintHappy = new FastTap(document);

})(Zepto);
